/*

------------------------------------------------------------------------------

A license is hereby granted to reproduce this software source code and
to create executable versions from this source code for personal,
non-commercial use.  The copyright notice included with the software
must be maintained in all copies produced.

THIS PROGRAM IS PROVIDED "AS IS". THE AUTHOR PROVIDES NO WARRANTIES
WHATSOEVER, EXPRESSED OR IMPLIED, INCLUDING WARRANTIES OF
MERCHANTABILITY, TITLE, OR FITNESS FOR ANY PARTICULAR PURPOSE.  THE
AUTHOR DOES NOT WARRANT THAT USE OF THIS PROGRAM DOES NOT INFRINGE THE
INTELLECTUAL PROPERTY RIGHTS OF ANY THIRD PARTY IN ANY COUNTRY.

Copyright (c) 1994-2018, John Conover, All Rights Reserved.

Comments and/or bug reports should be addressed to:

    john@email.johncon.com (John Conover)

------------------------------------------------------------------------------

sony.timing.c is for changing the Sony timing values in a file derived
from the Arduino IRLib2 library, AnalysIR sketch, to the industry
"standard" timing values. The input file structure is a text file
consisting of records of white space delimited timing values, in
temporal order, with the '-' characters removed. The "standard" values
are printed to stdout.

The Sony remote protocol is based on a base timing value of 600 uS,
and the timing values for a record are:

    1) A Mark of 4 X 600 uS = 2.4 mS
    2) Data Logical '0' a Space of 1 * 600 uS followed
       by a Mark of 1 * 600 uS
    3) Data Logical '1' a Space of 1 * 600 uS followed
       by a Mark of 2 * 600 uS
    4) A Record interval is always 75 * 600 uS = 45 mS
    5) A Record is sent three times

This simplifies the changing of the Sony timing values derived from the
IRLib2 library, AnalysIR sketch, which can be rounded to the closest
600 uS value.

$Revision: 0.0 $
$Date: 2018/01/18 19:36:00 $
$Id: sony.timing.c,v 0.0 2018/01/18 19:36:00 john Exp $
$Log: sony.timing.c,v $
Revision 0.0  2018/01/18 19:36:00  john
Initial version


*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <unistd.h>

#ifdef __STDC__

#include <float.h>

#endif

static char rcsid[] = "$Id: sony.timing.c,v 0.0 2018/01/18 19:36:00 john Exp $"; /* program version */
static char copyright[] = "Copyright (c) 1994-2018, John Conover, All Rights Reserved"; /* the copyright banner */

#define BUFLEN BUFSIZ /* i/o buffer size */

#define BASE 600 /* the base timing value for the Sony protocol is 600 uS */

#define TOKEN_SEPARATORS " \t\n\r\b\\,-{};" /* file record field separators */

#ifdef __STDC__

static const char *help_message[] = /* help message index array */

#else

static char *help_message[] = /* help message index array */

#endif

{
    "\n",
    "Change the Sony timing values in a file to industry standard values\n",
    "Usage: sony.timing [-V] [-v] [filename]\n",
    "    -V, verbose output\n",
    "    -v, print the program's version information\n",
    "    filename, input filename\n"
};

#ifdef __STDC__

static const char *error_message[] = /* error message index array */

#else

static char *error_message[] = /* error message index array */

#endif

{
    "No error\n",
    "Error in program argument(s)\n",
    "Error opening file\n",
    "Error closing file\n"
};

#define NOERROR 0 /* error values, one for each index in the error message array */
#define EARGS 1
#define EOPEN 2
#define ECLOSE 3

#ifdef __STDC__

static void print_message (int retval); /* print any error messages */
static int strtoken (char *string, char *parse_array, char **parse, char *delim);

#else

static void print_message (); /* print any error messages */
static int strtoken ();

#endif

#ifdef __STDC__

int main (int argc, char *argv[])

#else

int main (argc, argv)
int argc;
char *argv[];

#endif

{
    char buffer[BUFLEN], /* i/o buffer */
         parsebuffer[BUFLEN], /* parsed i/o buffer */
         *token[BUFLEN / 2], /* reference to tokens in parsed i/o buffer */
         token_separators[] = TOKEN_SEPARATORS;

    int retval = NOERROR, /* return value, assume no error */
        fields, /* number of fields in a record */
        i, /* field counter of fields in a record */
        value, /* timing value of a field in a record */
        nominal, /* closest integer value of the nominal timing value of a field in a record */
        N, /* integer value of base timing encode number of a field in a record, base = BASE uS */
        verbose = 0, /* verbose output, 0 = no, 1 = yes */
        c; /* command line switch */

    FILE *infile = stdin; /* reference to input file */

    while ((c = getopt (argc, argv, "Vv")) != EOF) /* for each command line switch */
    {

        switch (c) /* which switch? */
        {

            case 'V':

                verbose = 1; /* set the verbose output, 0 = no, 1 = yes */
                break;

            case 'v':

                (void) printf ("%s\n", rcsid); /* print the version */
                (void) printf ("%s\n", copyright); /* print the copyright */
                optind = argc; /* force argument error */
                retval = EARGS; /* assume not enough arguments */

            default: /* illegal switch? */

                optind = argc; /* force argument error */
                retval = EARGS; /* assume not enough arguments */
                break;
        }

    }

    if (retval == NOERROR) /* enough arguments? */
    {
        retval = EOPEN; /* assume error opening file */

        if ((infile = argc <= optind ? stdin : fopen (argv[optind], "r")) != (FILE *) 0) /* yes, open the input file */
        {
            retval = NOERROR; /* assume no error */

            while (fgets (buffer, BUFLEN, infile) != (char *) 0) /* read the records from the input file */
            {

                if ((fields = strtoken (buffer, parsebuffer, token, token_separators)) != 0) /* parse the record into fields, skip the record if there are no fields */
                {

                    if (token[0][0] != '#') /* if the first character of the first field is a '#' character, skip it */
                    {

                        for (i = 0; i < fields; i ++) /* for each field in the record */
                        {
                            value = atoi (token[i]); /* save the timing value of a field in a record */

                            /*

                            closest base timing value rounded to
                            nearest integer value.

                            */

                            N = (int) round (value / (double) BASE); /* save the integer value of base timing encode number of a field in a record, base = BASE uS */

                            /*

                            closest output timing value rounded to
                            nearest integer value.

                            */

                            nominal = (int) round ((double) N * (double) BASE); /* save the closest integer value of the nominal timing value of a field in a record */

                            if (verbose == 1) /* verbose output, 0 = no, 1 = yes, set? */
                            {
                                (void) printf ("%d, (%d, %d = %f%%)\n", nominal, value, value - nominal, (double) (((((double) value - (double) nominal) / (double) nominal) * (double) 100))); /* print the nominal timing value, the recorded value, the difference, and the percentage of the difference */
                            }

                            else
                            {
                                (void) printf ("%d\n", nominal); /* print the nominal timing value */
                            }

                        }

                    }

                }

            }

            if (argc > optind) /* using stdin as input? */
            {

                if (fclose (infile) == EOF) /* no, close the input file */
                {
                    retval = ECLOSE; /* error closing file */
                }

            }

        }

    }

    print_message (retval); /* print any error messages */
    exit (retval); /* exit with the error value */

#ifdef LINT

    return (0); /* for lint formalities */

#endif

}

/*

Print any error messages.

static void print_message (int retval);

I) Data structures:

    A) The help_message array is an array of character stings, one per
       line to be printed for help.

    B) The error_message array is an array of character strings, one
       line per error; the error_message array is implicitly addressed
       by the value integer retval, which specifies the error to be
       printed.

II) Function execution:

    A) Depending on the value of the integer retval:

        1) If retval is zero, print nothing-normal/successful program
           exit.

        2) Else if retval is unity, print help.

        3) Else retval is not zero or unity, it is an error code, print
           the corresponding error message.

Returns nothing.

*/

#ifdef __STDC__

static void print_message (int retval)

#else

static void print_message (retval)
int retval;

#endif

{
    size_t help_ctr; /* help_message line counter */

    switch (retval) /* which return value */
    {

        case 0: /* program ended without errors, print nothing */

            break;

        case 1: /* program ended with a request for help, or argument error, print help */

            for (help_ctr = 0; help_ctr < (sizeof (help_message) / sizeof (char *)); help_ctr ++) /* for each line of help */
            {
                (void) printf ("%s", help_message[help_ctr]); /* print the line */
            }

            break;

        default: /* an error that was not a request for help, print the error */

            (void) fprintf (stderr, "%s", error_message[retval]);
            break;

    }

}

/*

parse a record based on sequential delimiters

int strtoken (char *string, char *parse_array, char **parse, char *delim);

parse a character array, string, into an array, parse_array, using
consecutive characters from delim as field delimiters, point the
character pointers, token, to the beginning of each field, return the
number of fields parsed

*/

#ifdef __STDC__

static int strtoken (char *string, char *parse_array, char **parse, char *delim)

#else

static int strtoken (string, parse_array, parse, delim)
char *string,
*parse_array,
**parse,
*delim;

#endif

{
    int tokens = 0;

    (void) strcpy (parse_array, string); /* copy the string */

    parse[tokens] = strtok (parse_array, delim); /* get the 1st field */

    while (parse[tokens] != 0) /* get the remaining fields */
    {
        parse[++ tokens] = strtok ((char *) 0, delim);
    }

    return (tokens); /* return the number of tokens parsed */
}
